#include "WebNode.h"
#include <iomanip.h>

#define _BSD_SIGNALS
#include <signal.h>

#define MAXLEN 1024       // buffer length
#define OK_CODE "200"     // status code for OK in HTTP/1.x
#define MPERM_CODE "301"  // status code for Moved Perm in HTTP/1.x
#define MTEMP_CODE "302"  // status code for Moved Temp in HTTP/1.x
#define CRLF  "\015\012"

Cache WebNode::cache;

void
WebNode::download( const char* location )
  {
     char buf[MAXLEN];

     URL url( location );
     iosockinet  sio( sockbuf::sock_stream );
     //     sigblock( sigmask( SIGIO ) );
     if( sio->connect( url.getHostName(), url.getPortNum() ) != 0 )
       {
	  //sigsetmask( 0 );
	  throwMsg( "Connecting failed", "Unable to connect to server\nThe server may be down or unreachable" );
	  return;
       }
#ifdef DEBUG
     cerr << "local port = " << sio->localport() << '\t'
	<< "peer host = " << sio->peerhost() << endl;
#endif
     sio << "GET " << url.getResourceName() << " HTTP/1.0" << CRLF << CRLF;
     sio.flush();
     sio.getline( buf, MAXLEN );
     if( strstr( buf, MPERM_CODE ) || strstr( buf, MTEMP_CODE ) )
       {
	  cerr << "Resource moved - searching...\n";
	  while( sio.getline( buf, MAXLEN, '\r' ) )
	     if( strstr( buf, "Location:" ) )
	       {
		  //sigsetmask( 0 );
		  download( strchr( buf, ':' )+2 );
		  return;
	       }
       }
     if( !strstr( buf, OK_CODE ) )
       {
	  //sigsetmask( 0 );
	  throwMsg( "Downloading failed", buf );
	  return;
       }

     // checking 'Content-length: ' field
     dataSize = 0;
     while( sio.getline( buf, MAXLEN ) )
	if( strstr( buf, "Content-length:" ) )
	  {
	     dataSize = atoi( strchr( buf, ':' )+1 );
	     break;
	  }

     if( dataSize == 0 )
       {
	  //sigsetmask( 0 );
	  throwMsg( "Protocol failure", "Header does not comply with spec HTTP\nThere should be 'Content-length' field" );
	  return;
       }

     if( !( dataBuf = new char[ dataSize ] ) )
       {
	  throwMsg( "WebNode", "No memory for buffer" );
	  exit(1);
       }

     //discarding rest of HTTP header
     while( sio.getline( buf, MAXLEN ) )
	if( ! strcmp( buf, "\r" ) )
	   break;

     //reading actual data
     sio.read( dataBuf, dataSize );
     //sigsetmask( 0 );
 }
